<?php
/**
 * Jonah_RSSParse.
 *
 * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * $Horde: jonah/lib/RSSParse.php,v 1.8 2005/01/03 14:35:07 jan Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.8 $
 * @since   Jonah 0.0.1
 * @package Jonah
 */
class Jonah_RSSParse {

    /** Boolean indicating whether or not we should shout about
        unrecognized tags, or just go on. */
    var $debug = false;

    /** The Resource ID of the xml parser. */
    var $parser;

    /** The current parent tag - CHANNEL, STORY, etc. */
    var $parent = '';

    /** The current child tag - TITLE, DESCRIPTION, URL, etc. */
    var $child = '';

    /** All the attributes of the channel description. */
    var $channel;

    /** All the attributes of the channel image. */
    var $image;

    /** All the attributes of the current story being parsed. */
    var $story;

    /** All the attributes of the current item being parsed. */
    var $item;

    /** The array that all the parsed information gets dumped into. */
    var $structure;

    /** Holds an error string, if any. */
    var $error;

    /**
     * Constructs a new Jonah_RSSParse parser object.
     */
    function Jonah_RSSParse()
    {
        $this->channel = array();
        $this->image = array();
        $this->item = array();
        $this->story = array();
    }

    /**
     * Initialize the XML parser.
     */
    function init()
    {
        // Create the XML parser.
        $this->parser = xml_parser_create();
        xml_set_object($this->parser, $this);
        xml_parser_set_option($this->parser, XML_OPTION_CASE_FOLDING, true);
        xml_parser_set_option($this->parser, XML_OPTION_TARGET_ENCODING, 'UTF-8');
        xml_set_element_handler($this->parser, 'startElement', 'endElement');
        xml_set_character_data_handler($this->parser, 'characterData');
        xml_set_default_handler($this->parser, 'defaultHandler');

        // Disable processing instructions and external entities.
        xml_set_processing_instruction_handler($this->parser, '');
        xml_set_external_entity_ref_handler($this->parser, '');
    }

    /**
     * Clean up any existing data - reset to a state where we can
     * cleanly open a new file.
     */
    function cleanup()
    {
        $this->channel = array();
        $this->image = array();
        $this->item = array();
        $this->story = array();
        $this->structure = array();
    }

    /**
     * Actually do the parsing. Separated from the constructor just in
     * case you want to set any other options on the parser, load
     * initial data, whatever.
     *
     * @param $data  The XML data to parse as RSS.
     */
    function parse($data)
    {
        $this->init();

        // Sanity checks.
        if (!$this->parser) {
            $this->error = 'Could not find xml parser handle';
            return false;
        }

        // Parse.
        if (!@xml_parse($this->parser, $data)) {
            $this->error = sprintf('XML error: %s at line %d', xml_error_string(xml_get_error_code($this->parser)), xml_get_current_line_number($this->parser));
            return false;
        }

        // Clean up.
        xml_parser_free($this->parser);
    }

    /**
     * Start collecting data about a new element.
     */
    function startElement($parser, $name, $attribs)
    {
        switch ($name) {
        case 'CHANNEL':
        case 'IMAGE':
        case 'ITEM':
        case 'TEXTINPUT':
            $this->parent = $name;
            break;

        case 'LINUXTODAY':
        case 'UUTISET':
            $this->parent = 'channel';
            break;

        case 'UUTINEN':
            $this->parent = 'item';
            break;

        case 'STORYLIST':
            $this->structure['type'] = 'storylist';
            break;

        case 'RATING':
        case 'DESCRIPTION':
        case 'LINK':
        case 'WIDTH':
        case 'HEIGHT':
        case 'LANGUAGE':
        case 'NAME':
        case 'MANAGINGEDITOR':
        case 'WEBMASTER':
        case 'COPYRIGHT':
        case 'LASTBUILDDATE':
        case 'AUTHOR':
        case 'TOPIC':
        case 'COMMENTS':
            // Userland story format.
        case 'POSTTIME':
        case 'CHANNELTITLE':
        case 'CHANNELURL':
        case 'USERLANDCHANNELID':
        case 'STORYTEXT':
            $this->child = String::lower($name);
            break;

            // If we're inside an ITEM, consider this a LINK.
        case 'URL':
            if ($this->parent == 'item') {
                $this->child = 'link';
            } else {
                $this->child = 'url';
            }
            break;

            // For My.Userland channels, let these be STORY tags;
            // otherwise, map them to ITEMs.
        case 'STORY':
            if ($this->parent == 'storylist') {
                $this->parent = 'story';
            } else {
                $this->parent = 'item';
            }
            break;

            // Nonstandard bits that we want to map to standard bits.
        case 'TITLE':
        case 'OTSIKKO':
            $this->child = 'title';
            break;

        case 'PUBDATE':
        case 'TIME':
        case 'PVM':
            $this->child = 'pubdate';
            break;

        case 'RDF:RDF':
        case 'RSS':
            $this->child = 'junk';
            break;

        default:
            $this->child = 'junk';
            if ($this->debug) {
                echo 'found something unexpected: <b>' . htmlspecialchars($name) . '</b>. fix this.<br>' . "\n";
            }
            break;
        }
    }

    /**
     * Handle the ends of XML elements - wrap up whatever we've been
     * putting together and store it for safekeeping.
     */
    function endElement($parser, $name)
    {
        switch ($name) {
        case 'CHANNEL':
            $this->structure['channel'] = $this->channel;
            break;

        case 'IMAGE':
            $this->structure['image'] = $this->image;
            break;

        case 'STORY':
            if ($this->parent == 'storylist') {
                $this->structure['stories'][] = $this->story;
                $this->story = array();
            } else {
                $this->structure['items'][] = $this->item;
                $this->item = array();
            }
            break;

        case 'TEXTINPUT':
            $this->item['textinput'] = true;
            // No break here; continue to the next case.

        case 'UUTINEN':
        case 'ITEM':
            $this->structure['items'][] = $this->item;
            $this->item = array();
            break;
        }
    }

    /**
     * The handler for character data encountered in the XML file.
     */
    function characterData($parser, $data)
    {
        if (preg_match('|\S|', $data)) {
            switch ($this->parent) {
            case 'CHANNEL':
                if (isset($this->channel[$this->child])) {
                    $this->channel[$this->child] .= $data;
                } else {
                    $this->channel[$this->child] = $data;
                }
                break;

            case 'IMAGE':
                if (isset($this->image[$this->child])) {
                    $this->image[$this->child] .= $data;
                } else {
                    $this->image[$this->child] = $data;
                }
                break;

            case 'STORY':
                if (isset($this->story[$this->child])) {
                    $this->story[$this->child] .= $data;
                } else {
                    $this->story[$this->child] = $data;
                }
                break;

            default:
                if (isset($this->item[$this->child])) {
                    $this->item[$this->child] .= $data;
                } else {
                    $this->item[$this->child] = $data;
                }
                break;
            }
        }
    }

    /**
     * Handles things that we don't recognize. A no-op.
     */
    function defaultHandler($parser, $data)
    {
    }

}
